#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import io
import os

from PyQt5 import uic
from PyQt5.QtCore import pyqtProperty, QFile, QFileInfo, QTextStream
from PyQt5.QtWidgets import QWidget, QStackedLayout, QLayout 
from PyQt5.QtGui import QColor


import epyqlib.widgets.scale

# See file COPYING in this source tree
__copyright__ = 'Copyright 2017, EPC Power Corp.'
__license__ = 'GPLv2+'

class DualScale(QWidget):

    def __init__(self, parent=None, in_designer=False):

        QWidget.__init__(self, parent = parent)

        self.in_designer = in_designer

        self.d_vertically_flipped = False


        ui = self.getPath()

        # TODO: CAMPid 9549757292917394095482739548437597676742
        if not QFileInfo(ui).isAbsolute():
            ui_file = os.path.join(
                QFileInfo.absolutePath(QFileInfo(__file__)), ui)
        else:
            ui_file = ui
        ui_file = QFile(ui_file)
        ui_file.open(QFile.ReadOnly | QFile.Text)
        ts = QTextStream(ui_file)
        sio = io.StringIO(ts.readAll())
        self.ui = uic.loadUi(sio, self)

        self.scale1 = epyqlib.widgets.scale.Scale(self, in_designer)
        self.scale2 = epyqlib.widgets.scale.Scale(self, in_designer)
        # color ranges, scale markers, labels, needle painted.
        self.scale1.scale.m_paintMode = 1
        # needle, cover painted.
        self.scale2.scale.m_paintMode = 3
        # scale2's needle is blue
        self.scale2.scale.isBlue = True
        self.stackedLayout = QStackedLayout()
        self.stackedLayout.addWidget(self.scale2)
        self.stackedLayout.addWidget(self.scale1)
        self.stackedLayout.setStackingMode(1)
        self.ui.glayout.addLayout(self.stackedLayout, 0, 0)

        # Trying to figure out how to get lower min and the higher max and change the scale
        # true_min = min(self.scale1.ui.scale.m_minimum, self.scale2.ui.scale.m_minimum)
        # true_max = max(self.scale1.ui.scale.m_maximum, self.scale2.ui.scale.m_maximum)

        # self.scale1.minimum = true_min
        # self.scale2.minimum = true_min

        # self.scale1.maximum = true_max
        # self.scale2.maximum = true_max

        # self.scale1.override_range = True
        # self.scale2.override_range = True

    def getPath(self):
        return os.path.join(QFileInfo.absolutePath(QFileInfo(__file__)),
                          'dualscale.ui')

    @pyqtProperty('QString')
    def scale1_signal_path(self):
        return self.scale1.signal_path

    @scale1_signal_path.setter
    def scale1_signal_path(self, value):
        self.scale1.signal_path = value

    @pyqtProperty(bool)
    def scale1_label_visible(self):
        return self.scale1.label_visible

    @scale1_label_visible.setter
    def scale1_label_visible(self, new_visible):
        self.scale1.label_visible = new_visible


    @pyqtProperty('QString')
    def scale2_signal_path(self):
        return self.scale2.signal_path

    @scale2_signal_path.setter
    def scale2_signal_path(self, value):
        self.scale2.signal_path = value

    @pyqtProperty(bool)
    def scale2_label_visible(self):
        return self.scale2.label_visible

    @scale2_label_visible.setter
    def scale2_label_visible(self, new_visible):
        self.scale2.label_visible = new_visible

    @pyqtProperty(bool)
    def d_flipped(self):
        return self.d_vertically_flipped

    @d_flipped.setter
    def d_flipped(self, value):
        self.d_vertically_flipped = value
        self.scale1.s_flipped = value
        self.scale2.s_flipped = value



    @pyqtProperty(float)
    def lower_red_breakpoint(self):
        return self.scale1._breakpoints[0]

    @lower_red_breakpoint.setter
    def lower_red_breakpoint(self, breakpoint):
        self.scale1._breakpoints[0] = breakpoint
        self.scale1.update_configuration()

    @pyqtProperty(float)
    def lower_yellow_breakpoint(self):
        return self.scale1._breakpoints[1]

    @lower_yellow_breakpoint.setter
    def lower_yellow_breakpoint(self, breakpoint):
        self.scale1._breakpoints[1] = breakpoint
        self.scale1.update_configuration()

    @pyqtProperty(float)
    def upper_yellow_breakpoint(self):
        return self.scale1._breakpoints[2]

    @upper_yellow_breakpoint.setter
    def upper_yellow_breakpoint(self, breakpoint):
        self.scale1._breakpoints[2] = breakpoint
        self.scale1.update_configuration()

    @pyqtProperty(float)
    def upper_red_breakpoint(self):
        return self.scale1._breakpoints[3]

    @upper_red_breakpoint.setter
    def upper_red_breakpoint(self, breakpoint):
        self.scale1._breakpoints[3] = breakpoint
        self.scale1.update_configuration()

    @pyqtProperty(QColor)
    def lower_red_color(self):
        return self.scale1._colors[0]

    @lower_red_color.setter
    def lower_red_color(self, color):
        self.scale1._colors[0] = color
        self.scale1.update_configuration()

    @pyqtProperty(QColor)
    def lower_yellow_color(self):
        return self.scale1._colors[1]

    @lower_yellow_color.setter
    def lower_yellow_color(self, color):
        self.scale1._colors[1] = color
        self.scale1.update_configuration()

    @pyqtProperty(QColor)
    def green_color(self):
        return self.scale1._colors[2]

    @green_color.setter
    def green_color(self, color):
        self.scale1._colors[2] = color
        self.scale1.update_configuration()

    @pyqtProperty(QColor)
    def upper_yellow_color(self):
        return self.scale1._colors[3]

    @upper_yellow_color.setter
    def upper_yellow_color(self, color):
        self.scale1._colors[3] = color
        self.scale1.update_configuration()

    @pyqtProperty(QColor)
    def upper_red_color(self):
        return self.scale1._colors[4]

    @upper_red_color.setter
    def upper_red_color(self, color):
        self.scale1._colors[4] = color
        self.scale1.update_configuration()
 


#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import functools
import io
import os
import textwrap
import time

from PyQt5 import uic
from PyQt5.QtCore import pyqtProperty, QFile, QFileInfo, QTextStream
from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QLabel, QHBoxLayout, QWidget

# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


class ParameterEdit(QWidget):
    def __init__(self, parent=None, in_designer=False, edit=None, nv=None,
                 dialog=None, esc_action=None):
        QWidget.__init__(self, parent=parent)

        self.in_designer = in_designer

        ui = os.path.join(QFileInfo.absolutePath(QFileInfo(__file__)),
                          'parameteredit.ui')

        # TODO: CAMPid 9549757292917394095482739548437597676742
        if not QFileInfo(ui).isAbsolute():
            ui_file = os.path.join(
                QFileInfo.absolutePath(QFileInfo(__file__)), ui)
        else:
            ui_file = ui
        ui_file = QFile(ui_file)
        ui_file.open(QFile.ReadOnly | QFile.Text)
        ts = QTextStream(ui_file)
        sio = io.StringIO(ts.readAll())
        self.ui = uic.loadUi(sio, self)

        self._edit = edit
        self._dialog = dialog
        self.nv = nv

        self.ui.to_device.set_signal(self.nv)
        self.nv.status_signal.value_changed.connect(self.nv.value_changed)

        self.ui.save_to_nv_button.clicked.connect(self.save_to_nv)

        self.ui.to_device.edited.connect(self.edited)

        self.ui.description.setText(self.ui.to_device.toolTip())

        self.esc_action = esc_action
        self.ui.esc_button.clicked.connect(self.esc)

    def esc(self, checked):
        if self.esc_action is not None:
            self.esc_action()

    def edited(self, value):
        self.nv.read_from_device()
        time.sleep(0.05)
        self.nv.set_human_value(value)
        self.nv.write_to_device()

    def save_to_nv(self):
        focus_self = functools.partial(
            self.parent().setCurrentWidget,
            self
        )

        # TODO: CAMPid 93849811216123127753953680713426
        def inverter_to_nv():
            self.nv.tree_parent.module_to_nv()
            focus_self()

        self._dialog.focus(ok_action=inverter_to_nv,
                         cancel_action=focus_self,
                         label=textwrap.dedent('''\
                             Save all parameters to NV?''')
                         )

if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import epyqlib.widgets.abstractwidget
import os
import re

from PyQt5 import QtCore
from PyQt5.QtCore import pyqtProperty, QFile, QFileInfo, QEvent
from PyQt5.QtGui import QColor
from PyQt5.QtXml import QDomDocument

# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


darker_factor = 400


class FontChangeEventSignal(QtCore.QObject):
    triggered = QtCore.pyqtSignal()

    def eventFilter(self, _, event):
        if event.type() == QtCore.QEvent.FontChange:
            self.triggered.emit()

        return False


def set_attribute_recursive(element, tag_name, attribute,
                            new_color):
    if element.tagName() == tag_name:
        text = element.attribute(attribute)
        text = re.sub('(?<=fill:#)[0-9A-F]{6}', new_color, text)
        element.setAttribute(attribute, text)

    child_nodes = element.childNodes()
    children = [child_nodes.at(i) for i in range(child_nodes.length())]
    for child in children:
        if child.isElement():
            set_attribute_recursive(element=child.toElement(),
                                    tag_name=tag_name,
                                    attribute=attribute,
                                    new_color=new_color)


def make_color(svg_string, new_color):
    doc = QDomDocument()
    doc.setContent(svg_string)
    set_attribute_recursive(element=doc.documentElement(),
                            tag_name="circle",
                            attribute="style",
                            new_color=new_color)

    return doc.toByteArray()


def rgb_string(color):
    return ('{:02X}' * 3).format(*color.getRgb())


class Led(epyqlib.widgets.abstractwidget.AbstractWidget):
    def __init__(self, parent=None, in_designer=False):
        ui_file = os.path.join(QFileInfo.absolutePath(QFileInfo(__file__)),
                               'led.ui')

        epyqlib.widgets.abstractwidget.AbstractWidget.__init__(self,
                ui=ui_file, parent=parent, in_designer=in_designer)

        file_name = 'led.svg'

        # TODO: CAMPid 9549757292917394095482739548437597676742
        if not QFileInfo(file_name).isAbsolute():
            file = os.path.join(
                QFileInfo.absolutePath(QFileInfo(__file__)), file_name)
        else:
            file = file_name
        file = QFile(file)
        file.open(QFile.ReadOnly | QFile.Text)
        self.svg_string = file.readAll()

        # TODO: shouldn't this be in AbstractWidget?
        self._on_value = 1
        self._relative_height = 1
        self._label_from_enumeration = False

        self._value = False
        self.svg = {
            'on': None,
            'automatic_off': None,
            'manual_off': None
        }
        self.ui.value.main_element = 'led'

        self._last_loaded_svg = object()

        self._on_color = QColor()
        self._manual_off_color = QColor()
        self._automatic_off_color = True
        self.on_color = QColor("#20C020")
        self.manual_off_color = self.on_color.darker(factor=darker_factor)

        self.update_svg()

        self.font_event_signal = FontChangeEventSignal()
        self.ui.label.installEventFilter(self.font_event_signal)
        self.font_event_signal.triggered.connect(self.update_svg)

    @pyqtProperty(bool)
    def label_from_enumeration(self):
        return self._label_from_enumeration

    @label_from_enumeration.setter
    def label_from_enumeration(self, from_enumeration):
        self._label_from_enumeration =  bool(from_enumeration)

        # TODO: this is a hacky way to trigger an update
        self.set_signal(signal=self.signal_object,
                        force_update=True)

    @pyqtProperty(int)
    def on_value(self):
        return self._on_value

    @on_value.setter
    def on_value(self, new_on_value):
        new_on_value = int(new_on_value)
        if self._on_value != new_on_value:
            self._on_value = new_on_value
            self.update_svg()

            # TODO: this is a hacky way to trigger an update
            self.set_signal(signal=self.signal_object,
                            force_update=True)

    @pyqtProperty(bool)
    def automatic_off_color(self):
        return self._automatic_off_color

    @automatic_off_color.setter
    def automatic_off_color(self, automatic):
        automatic = bool(automatic)
        if automatic != self._automatic_off_color:
            self._automatic_off_color = bool(automatic)
            self.update_svg()

    @pyqtProperty(QColor)
    def on_color(self):
        return self._on_color

    @on_color.setter
    def on_color(self, new_color):
        self._on_color = QColor(new_color)

        self.svg['on'] = make_color(self.svg_string, rgb_string(self._on_color))

        if self.automatic_off_color:
            self.svg['automatic_off'] = make_color(
                self.svg_string,
                rgb_string(self._on_color.darker(factor=darker_factor))
            )

        self.update_svg()

    @pyqtProperty(QColor)
    def manual_off_color(self):
        return self._manual_off_color

    @manual_off_color.setter
    def manual_off_color(self, new_color):
        self._manual_off_color = QColor(new_color)

        self.svg['manual_off'] = make_color(self.svg_string, rgb_string(self._manual_off_color))

        self.update_svg()

    @pyqtProperty(float)
    def relative_height(self):
        return self._relative_height

    @relative_height.setter
    def relative_height(self, multiplier):
        self._relative_height = multiplier

        self.update_svg()

    def set_value(self, value):
        # TODO: quit hardcoding this and it's better implemented elsewhere
        if self.signal_object is not None:
            value = self.signal_object.value
        else:
            value = value

        self._value = (value == self.on_value)

        self.update_svg()

    def update_svg(self):
        if self._value:
            svg = self.svg['on']
        elif self.automatic_off_color:
            svg = self.svg['automatic_off']
        else:
            svg = self.svg['manual_off']

        if svg is not self._last_loaded_svg:
            self.ui.value.load(svg)
            self._last_loaded_svg = svg

        # TODO: figure out if this should be needed.  seems not but
        #       without it the font metric can return invalid values
        #       at least when the label_visible property is not
        #       explicitly set in the .ui file and the font size
        #       has been set via a second stylesheet.
        #
        #       Possibly:
        #           https://bugreports.qt.io/browse/QTBUG-42720

        self.ui.label.setVisible(self.ui.label.isVisibleTo(self))
        height = self.relative_height * self.ui.label.fontMetrics().height()

        ratio = self.ui.value.ratio()
        if ratio != 0:
            width = height / ratio
        else:
            width = 0

        self.ui.value.setFixedSize(width, height)

    def set_label_custom(self, new_signal=None):
        label = None

        if new_signal is not None:
            if self.label_from_enumeration and new_signal is not None:
                label = new_signal.enumeration[self.on_value]

        return label

    def event(self, *args, **kwargs):
        result = epyqlib.widgets.abstractwidget.AbstractWidget.event(
            self, *args, **kwargs
        )

        event = args[0]
        if event.type() == QEvent.Polish:
            self.update_svg()

        return result

    def set_unit_text(self, units):
        pass


if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure



import io
import os
from PyQt5 import QtWidgets, uic
from PyQt5.QtCore import (pyqtProperty, pyqtSlot, Qt, QFile,
                          QFileInfo, QTextStream)
from PyQt5.QtGui import QFontMetrics

# See file COPYING in this source tree
__copyright__ = 'Copyright 2018, EPC Power Corp.'
__license__ = 'GPLv2+'


styles = {
    'red': "background-color: rgba(255, 255, 255, 0);"
                           "color: rgba(255, 85, 85, 25);",
    'blue': "background-color: rgba(255, 255, 255, 0);"
                           "color: rgba(85, 85, 255, 25);"
}


class OverlayLabel(QtWidgets.QWidget):
    def __init__(self, parent=None, in_designer=False):
        QtWidgets.QWidget.__init__(self, parent=parent)

        self.in_designer = in_designer

        ui = 'overlaylabel.ui'
        # TODO: CAMPid 9549757292917394095482739548437597676742
        if not QFileInfo(ui).isAbsolute():
            ui_file = os.path.join(
                QFileInfo.absolutePath(QFileInfo(__file__)), ui)
        else:
            ui_file = ui
        ui_file = QFile(ui_file)
        ui_file.open(QFile.ReadOnly | QFile.Text)
        ts = QTextStream(ui_file)
        sio = io.StringIO(ts.readAll())
        self.ui = uic.loadUi(sio, self)

        self.setStyleSheet(styles['red'])

        self.ui.setAttribute(Qt.WA_TransparentForMouseEvents)

        self._width_ratio = 0.8
        self._height_ratio = 0.8

    @pyqtProperty(str)
    def text(self):
        self.ui.label.text()

    @text.setter
    def text(self, text):
        self.ui.label.setText(text)

    @pyqtProperty(float)
    def width_ratio(self):
        return self._width_ratio

    @width_ratio.setter
    def width_ratio(self, value):
        self._width_ratio = value

    @pyqtProperty(float)
    def height_ratio(self):
        return self._height_ratio

    @height_ratio.setter
    def height_ratio(self, value):
        self._height_ratio = value

    def resizeEvent(self, event):
        QtWidgets.QWidget.resizeEvent(self, event)

        self.update_overlay_size(event.size())

    def update_overlay_size(self, size):
        text = self.label.text()
        if not text:
            text = '-'
        font = self.label.font()
        font.setPixelSize(1000)
        metric = QFontMetrics(font)
        rect = metric.boundingRect(text)

        pixel_size_width = (
            font.pixelSize() *
            (size.width() * self.width_ratio) / rect.width()
        )

        pixel_size_height = (
            font.pixelSize() *
            (size.height() * self.height_ratio) / rect.height()
        )

        self.ui.label.setStyleSheet('font-size: {}px; font-weight: bold'.format(
            round(min(pixel_size_width, pixel_size_height))))



# This file has been generated


build_system = 'AppVeyor'
build_id = '26241728'
build_number = '339'
build_version = '1.0.339'
job_id = '8ivvlj5eqd3hwtta'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/8ivvlj5eqd3hwtta'



#!/usr/bin/env python3

#TODO: """DocString if there is one"""

import epyqlib.listmenu
import functools
import io
import os

from PyQt5 import uic
from PyQt5.QtCore import pyqtProperty, QFile, QFileInfo, QTextStream
from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QLabel, QHBoxLayout, QWidget

# See file COPYING in this source tree
__copyright__ = 'Copyright 2016, EPC Power Corp.'
__license__ = 'GPLv2+'


class ListSelect(QWidget):
    def __init__(self, parent=None, in_designer=False, action=None):
        QWidget.__init__(self, parent=parent)

        self.in_designer = in_designer

        ui = os.path.join(QFileInfo.absolutePath(QFileInfo(__file__)),
                          'listselect.ui')

        # TODO: CAMPid 9549757292917394095482739548437597676742
        if not QFileInfo(ui).isAbsolute():
            ui_file = os.path.join(
                QFileInfo.absolutePath(QFileInfo(__file__)), ui)
        else:
            ui_file = ui
        ui_file = QFile(ui_file)
        ui_file.open(QFile.ReadOnly | QFile.Text)
        ts = QTextStream(ui_file)
        sio = io.StringIO(ts.readAll())
        self.ui = uic.loadUi(sio, self)

        self.action = action
        self.items = {}
        self.ui.accept_button.clicked.connect(self.accept)
        self.ui.cancel_button.clicked.connect(
            functools.partial(self.exit, value=None))

    def focus(self, value, action, items, label=''):
        self.items = items
        root = epyqlib.listmenu.Node(text=label)
        model = epyqlib.listmenu.ListMenuModel(root=root)
        self.ui.menu_view.setModel(model)

        selected = None

        for key, item_value in sorted(self.items.items()):
            node = epyqlib.listmenu.Node(text=item_value, action=lambda: None)
            root.append_child(node)

            if key == int(value):
                selected = node

        if selected is not None:
            self.ui.menu_view.select_node(selected)

        self.action = action
        parent = self.parent()
        if hasattr(parent, 'setCurrentWidget'):
            parent.setCurrentWidget(self)
        focused_widget = self.focusWidget()
        if focused_widget is not None:
            focused_widget.clearFocus()

    def accept(self):
        selected = self.ui.menu_view.selected_text()
        found_key = None
        for key, value in self.items.items():
            if value == selected:
                found_key = key
                break

        self.exit(value=found_key)

    def exit(self, value):
        self.action(value=value)

if __name__ == '__main__':
    import sys

    print('No script functionality here')
    sys.exit(1)     # non-zero is a failure
